C
C
C PROGRAM NAME:		FORMLINE
C
C
C PURPOSE:		PROGRAM MAY BE USED TO REFORMAT FORTRAN PROGRAMS
C			THAT HAVE ORIGINATED IN THE FORM OF 80 CHARACTER
C			CARD IMAGES INTO THE FORM PRODUCED BY THE VAX 11/780
C			EDITOR, THAT IS VARIABLE LENGTH RECORDS WITH 
C			STATEMENT NUMBERS LEFT JUSTIFIED AND WITH BODY OF
C			LINE (COMMENT OR STATEMENT) BEGINNING AT FIRST
C			TAB STOP (COLUMN 9).
C
C
C			THE PROGRAM IS INTENDED TO BE ABLE TO DEAL WITH INPUT
C			FILES THAT CONTAIN ONLY 80 CHARACTER CARD IMAGES AS
C			WELL AS FILES WHICH MAY CONTAIN IN ADDITION LINES
C			PREPARED BY THE EDITOR.
C
C NOTES:		1.PROGRAM QUERIES FOR NAMES OF INPUT AND OUTPUT FILES
C
C			2.THE INPUT FILE MAY BE OF EITHER 'FIXED' RECORDTYPE
C			  SUCH AS PRODUCED BY THE "COPY" COMMAND OR OF
C			  'VARIABLE' PRODUCED BY THE EDITOR.
C
C			3. LINES OF THE INPUT FILE WHICH CONTAIN THE TAB
C			   CODE (09) ARE PRESUMED TO HAVE BEEN PREPARED
C			   BY THE EDITOR AND ARE WRITTEN UNCHANGED TO THE
C			   OUTPUT FILE.
C
C		        4. THE BODY OF A STATEMENT WILL NOT BE SHIFTED TO
C			   THE FIRST TAB STOP IF TO DO SO WILL CAUSE THE 
C			   STATEMENT TO OVERFLOW COLUMN 72.
C
C			5. A COMMENT WHOSE BODY BEGINS IN COLUMNS 2 THRU 6
C			   IS PRESUMED TO BE IN CORRECT SPOT AND IS NOT
C			   SHIFTED.
C 
C			6. A COMMENT WHOSE BODY AT COLUMN 7 OR ABOVE WILL
C			   BE SHIFTED TO FIRST TAB STOP IF TO DO SO WILL
C			   NOT OVERFLOW COLUMN 80.
C
C			7. BLANK AND ZERO LENGTH RECORDS ARE IGNORED.
C
C			8. THE PROGRAM QUERIES AS TO WHETHER USER WISHES
C			   TO DELETE  CARD ID CHARACTERS (COLUMNS 73-80)
C			   FROM THE LINE.
C
C			9. COMPILE/LINK
C				$FOR/LIS/CHECK=ALL FORMLINE.FOR
C				$LINK FORMLINE.OBJ
C
C AUTHOR:		THOMAS L. TERRALL
C
C	 - DECLARATIONS -
C
	IMPLICIT	INTEGER*2	(I,J,K,L,M,N)
	IMPLICIT	REAL*8		(A-H,O-Z)
C
	INTEGER*2	RCR_SIZE
C
	LOGICAL*1	IN(100),OUT(100)
C
	CHARACTER*1	CARD_ID
C
C
	CHARACTER*30	IN_NAME,OUT_NAME
C
	WRITE(6,*) '          # # # BEGIN FORMLINE # # #'
C
C	- READ INPUT AND OUTPUT FILE NAMES -
C
10	TYPE 12
   12	FORMAT(' INPUT FILE NAME(30A)')
	ACCEPT 16,IN_NAME
   16	FORMAT(30A)
	IF(IN_NAME .EQ. ' ') GO TO 900	!END PROGRAM?
C
	TYPE 18
   18	FORMAT(' OUTPUT FILE NAME(30A)')
	ACCEPT 16,OUT_NAME
C
C	- OPEN FILES -
C
	OPEN(UNIT=80,NAME=IN_NAME,TYPE='OLD',
	1	ACCESS='SEQUENTIAL',FORM='FORMATTED',
	2	ORGANIZATION='SEQUENTIAL',ERR=410)
	OPEN(UNIT=82,NAME=OUT_NAME,TYPE='NEW',ERR=420,
	1	ACCESS='SEQUENTIAL',FORM='FORMATTED',
	2	ORGANIZATION='SEQUENTIAL',RECORDTYPE='VARIABLE',
	3	CARRIAGECONTROL='LIST')
C
C	CARD ID DELETION DESIRED?
C
11	TYPE 14
14	FORMAT(' DO YOU WISH TO DELETE ID ON CARDS'
	1     ' (COLUMNS 73-80) (Y OR N)?')
	ACCEPT 13, CARD_ID
13	FORMAT(A)
	IF(CARD_ID .NE. 'Y' .AND. CARD_ID .NE. 'N')GO TO 11
C
	K=0
C
C
C   - READ RECORDS OF INPUT FILE AND WRITE TO OUTPUT FILE -
C
C	READ RECORD OF INPUT FILE INTO INPUT ARRAY (IN)
C
40	READ(80,250,ERR=430,END=550)RCR_SIZE,(IN(I),I=1,RCR_SIZE)
250	FORMAT(Q,100A)
C
C	BLANK OUTPUT ARRAY (OUT)
C
55	DO 20 I=1,100	
	OUT(I) = ' '
20	CONTINUE
	IF(RCR_SIZE .EQ. 0) GO TO 40	!ZERO LENGTH RECORD?
C
C	DETERMINE RIGHT LIMIT OF STATEMENT OR COMMENT (IE)
C
	DO 24 IE=RCR_SIZE,1,-1
	IF(IN(IE) .NE. ' ') GO TO 30
24	CONTINUE
	GO TO 40
30	K=K+1	!INCREMENT OUTPUT LINE NUMBER
C
C	IS THIS LINE FROM EDITOR (I.E. TAB CODE PRESENT)?
C
  	DO 32 I=1,IE
	IF(IN(I) .NE. 09) GO TO 32
	JE=0
	IS=1
	GO TO 58
32	CONTINUE
C
C	RELOCATE IE TO IGNORE CARD ID
C
	IF(CARD_ID .EQ. 'N') GO TO 34
	IF(IE .LE. 72) GO TO 34
	DO 90 IE=72,1,-1
	IF(IN(IE) .NE. ' ') GO TO 34
90	CONTINUE
	WRITE(6,91) K
91	FORMAT(' MISSHAPED CARD BEFORE OUTPUT LINE - 'I5,
	1       ' - IGNORED')
	K=K-1
	GO TO 40
C
C	STATEMENT OR COMMENT LINE?
C
34	IF(IN(1) .EQ. 'C') GO TO 70
C
C	DETERMINE START OF STATEMENT NUMBER (KS)
C
  	DO 42 KS=1,5
	IF(IN(KS) .NE. ' ') GO TO 44
42	CONTINUE
	JE=0
	GO TO 51
C
C	DETERMINE END OF STATEMENT NUMBER (KE)
C
44	IF(KS .NE. 5) GO TO 45
	KE=6
	GO TO 48
45	DO 46 KE=KS+1,5
	IF(IN(KE) .EQ. ' ') GO TO 48
46	CONTINUE
C
C	LEFT JUSTIFY STATEMENT NUMBER
C
48	KE=KE-1
	JE=0
	DO 50 I=KS,KE
	JE=JE+1
	OUT(JE)=IN(I)
50	CONTINUE
C
C	LOCATE START OF BODY OF STATEMENT (IS)
C
51	DO 52 IS=6,IE
	IF(IN(IS) .NE. ' ') GO TO 53
52	CONTINUE
	WRITE(6,91) K
	K=K-1
	GO TO 40
C
C	ENOUGH SPACE TO SHIFT BODY OF STATEMENT TO FIRST TAB?
C
53	IF(8+IE-IS+1 .LE. 72) GO TO 54 
	JE=IS-1
	GO TO 58	!NO-DO NOT SHIFT
C
C	YES - INSERT TAB CODE AFTER STATEMENT NUMBER
C
54	JE=JE+1
	OUT(JE)=09
C
C	MOVE BODY OF LINE FROM INPUT ARRAY TO OUTPUT ARRAY
C	WITH POSSIBLE SHIFT
C
58	DO 60 I=IS,IE
	JE=JE+1
	OUT(JE)=IN(I)
60	CONTINUE
	GO TO 150
C
C	LOCATE START OF BODY OF COMMENT
C
70	JE=1
	IS=2
	OUT(1)='C'
	IF(IE .EQ. 1)GO TO 150
	DO 72 IS=2,IE
	IF(IN(IS) .NE. ' ')GO TO 74
72	CONTINUE
	WRITE(6,*) ' PROGRAM FAILURE - ABEND THIS FILE'
	GO TO 560
C
C	DOES BODY OF COMMENT BEGIN IN COLUMNS 2 THRU 6?
C
74	IF(IS .LE. 6) GO TO 88  !DO NOT SHIFT
C
C	ENOUGH SPACE TO SHIFT BODY OF COMMENT?
C
80	IF(8+IE-IS+1 .GT. 80) GO TO 88 !NO-DO NOT SHIFT
C
C	YES - INSERT TAB CODE 
C
	OUT(2)=09
	JE=2
	GO TO 58
C
C	RESET INPUT LINE START FOR NO SHIFT
C
88	IS=2
	GO TO 58
C
C	WRITE LINE IN OUTPUT ARRAY TO OUTPUT FILE
150	WRITE(82,155,ERR=440)(OUT(I),I=1,JE)
155	FORMAT(100A)
	GO TO 40
C
C	- ERROR MESSAGES -
410	WRITE(6,*) ' ERROR OPENING INPUT FILE'
	GO TO 10
C
420	WRITE(6,*) ' ERROR OPENING OUTPUT FILE'
	GO TO 570
C
440	WRITE(6,*) ' OUTPUT FILE ACESS ERROR - ABEND'
	GO TO 560
C
430	WRITE(6,*) ' INPUT FILE ACCESS ERROR - ABEND'
	GO TO 560
C
540	WRITE(6,*) ' INPUT FILE COMPLETED - END'
	GO TO 550
C
C	 - CLOSE FILES -
550	CLOSE(UNIT=82)
	GO TO 570
560	CLOSE(UNIT=82,DISPOSE='DELETE')
570	CLOSE(UNIT=80)
	GO TO 10
C
C	END PROGRAM
600	WRITE(6,*) '          # # # END FORMLINE # # #'
900	CALL EXIT
	END
